#ifndef READING_HPP
#define READING_HPP

#include <iostream>
#include <fstream>
#include "trace_data.hpp"
using namespace std;

// constants specific to the SEG-Y standard
const int nTxtFileHeader = 3200; // textual file header length (bytes)
const int nBinFileHeader = 400; // binary file header length (bytes)
const int nTraceHeader = 240; // each trace header length (bytes)


// Check the length of the file (used when nsamples is really big)
int FileLength(const char * filename){
  int length_of_file_bytes = 0; // want to output how many bytes are in the file
  ifstream inputfile; // open file
  inputfile.open(filename, ios::in|ios::binary);
  if (inputfile){
    inputfile.seekg(0, inputfile.end); // get to position of file end 
    length_of_file_bytes = inputfile.tellg(); // record position number
    inputfile.close();
  }
  return length_of_file_bytes;
}


// get the header information that's critical from a SEG-Y
header_vals SEGYInfo(const char * filename){
  header_vals ThisFileInfo;
  short int nTraces, sampleFrq, shortNSamples;
  ifstream inputfile;

  inputfile.open(filename, ios::in|ios::binary); // open file
  if(inputfile){

    // get number of traces (number of channels)
    inputfile.seekg(3213,ios::beg);
    inputfile.read((char *)&nTraces,sizeof(short int));
    ThisFileInfo.nTraces = nTraces;

    // get the number of samples recorded per second
    inputfile.seekg(3217,ios::beg);
    inputfile.read((char *)&sampleFrq,sizeof(short int));
    ThisFileInfo.sampleFrq = sampleFrq;

    // get the number of samples recorded per channel
    inputfile.seekg(3221,ios::beg);
    inputfile.read((char *)&shortNSamples,sizeof(short int));
    // if too many samples recorded for standard, back-calc from file size
    if (shortNSamples == 0){
      int nbytes = FileLength(filename);
      int bytesPerChannel = int(float(nbytes - nTxtFileHeader - nBinFileHeader)/float(nTraces));
      int dataBytesPerChannel = bytesPerChannel - nTraceHeader;
      ThisFileInfo.nSamples = dataBytesPerChannel/sizeof(float);
    } else{
      ThisFileInfo.nSamples = int(shortNSamples);
    }

    inputfile.close();
  }

  return ThisFileInfo;
}

// byte swapping helper function
float byteSwapper(char * value){
  float valueSwapped;
  char * valueSwappedChars = (char *) &valueSwapped;
  valueSwappedChars[3] = value[0];
  valueSwappedChars[2] = value[1];
  valueSwappedChars[1] = value[2];
  valueSwappedChars[0] = value[3];

  return valueSwapped;
  }

// read a trace into an array of floats
void readTrace(const char * filename, double * trace_data , int trace_number, int number_samples, int start_sample, int number_samples_to_read){
  if(number_samples_to_read == 0){
    number_samples_to_read = number_samples;
  }
  ifstream inputfile;
  inputfile.open(filename, ios::in|ios::binary);
  if(inputfile){
    inputfile.seekg(nTxtFileHeader + nBinFileHeader + (nTraceHeader + number_samples*sizeof(float))*trace_number + nTraceHeader + start_sample*sizeof(float),ios::beg);
    char * readvalue;
    readvalue = new char [sizeof(float)]; 
    float value;
    for(int sample=0; sample < number_samples_to_read; sample++){
      inputfile.read(readvalue,sizeof(float));
      value = byteSwapper(readvalue);
      trace_data[sample] = double(value);
    }
    inputfile.close();
    delete[] readvalue;
  }

}


#endif
