#ifndef ARRAYOPS
#define ARRAYOPS

#include <complex.h>
#include <fftw3.h>
#include <cmath>
#include <algorithm>

void scaleArray(double * the_array, int array_length, double scale_factor){
  // multiply the next array_length elements of the_array by scale_factor
  for(int i=0; i<array_length; ++i){
    the_array[i] *= scale_factor;
  }
}

void multiplyArrays(fftw_complex * array1, fftw_complex * array2, fftw_complex * output, int array_length){
	// output[i] = array1[i] + array2[i]
	for(int i=0; i<array_length; ++i){
		output[i][0] = array1[i][0] * array2[i][0] - array1[i][1] * array2[i][1];
		output[i][1] = array1[i][0] * array2[i][1] + array1[i][1] * array2[i][0];
	}
}

void addToArray(double * the_array, int array_length, double addValue){
  // incrrease the next array_length elements of the_array by addValue
  for(int i=0; i<array_length; ++i){
    the_array[i] += addValue;
  }
}

void addArrays(double * array1, double * array2, double * sum, int nElements){
	// sum[i] = array1[i] + array2[i]
	// this is not optimized at all.
	for(int i=0; i<nElements; ++i){
		sum[i] = array1[i] + array2[i]; 
	}
}

void addArraysInPlace(double * anArray, double * arrayToAddOnTo, int nElements){
	// arrayToAddOnTo[i] = anArray[i] + arrayToAddOnTo[i]
	// this is not optimized.
	for(int i=0; i<nElements; ++i){
		arrayToAddOnTo[i] = anArray[i] + arrayToAddOnTo[i];
	}	
}


void addArraysInPlace(fftw_complex * anArray, fftw_complex * arrayToAddOnTo, int nElements){
	// arrayToAddOnTo[i] = anArray[i] + arrayToAddOnTo[i]
	// this is not optimized.
	for(int i=0; i<nElements; ++i){
		arrayToAddOnTo[i][0] = anArray[i][0] + arrayToAddOnTo[i][0];
		arrayToAddOnTo[i][1] = anArray[i][1] + arrayToAddOnTo[i][1];
	}	
}


double absQuantile(fftw_complex * the_array, int nElements, float proportionBigger){
  // find the absolute value of the entry in the_array that is smaller than
  // proportionBigger*nElements of the abs value of elements in the_array.
  double magnitudes[nElements];
  for (int i=0; i<nElements; ++i){
    magnitudes[i] = sqrt(the_array[i][0]*the_array[i][0]+the_array[i][1]*the_array[i][1]); // cabs(*(the_array[i])); // cabs was okay on Snowbear but not on SEP machines
  }
  std::sort(magnitudes, magnitudes+nElements);
  int quantileIndex = int((1.0-proportionBigger)*nElements);
  return magnitudes[quantileIndex]; 
}

double complexMultiplyRe(double re1, double im1, double re2, double im2){
  return re1*re2 - im1*im2;
}

double complexMultiplyIm(double re1, double im1, double re2, double im2){
  return re1*im2 + im1*re2;
}

double complexMultiplyAbs(double re1, double im1, double re2, double im2){
  return std::sqrt((re1*re1 + im1*im1) * (re2*re2 + im2*im2));
}

#endif
