#include <iostream>
#include <fstream>
#include <string>
#include <assert.h>
#include <vector>
#include <cstdlib>
#include <complex>
#include <fftw3.h>
#include "reading.hpp"
#include "trace_data.hpp"
#include "trace.hpp"
#include "filter_workflows.hpp"
using namespace std;

int main(int argc, char* argv[]) {
  try{
    // input file names
    char *fileListName = argv[1]; // input file with list of file names to use
    ifstream fileList(fileListName);
    string filenameStr;


    // channels of interest
    int startChannel = atoi(argv[2]);
    int endChannel = atoi(argv[3]); 
    int nChannels = endChannel-startChannel+1;
    int srcChannel = int((endChannel+startChannel)/2.0);
    // number of time lags of interest
    int nLags = atoi(argv[4]);
    // frequencies (Hz) of bandpass 
    float minFrq = 3.0;
    float maxFrq = 50.0;
    
    // create array to hold cross-correlations (fast direction is nLags long)
    double * xcorrs = new double[nLags*nChannels];
    for(int i=0; i<nLags*nChannels; ++i){
      xcorrs[i] = 0;
    }
    
    // open the output text file where beamforming results will be printed
    char *outfileName = argv[5];
    ofstream outfile(outfileName);

    if(fileList.is_open()){ 
      std::cout << "files open \n";
        while(getline(fileList,filenameStr)){
	
		// read relevant traces from this file
		const char *filename = filenameStr.c_str();
		cout << filename << "\n";
		header_vals ThisFileInfo = SEGYInfo(filename); // get nTraces, sampleFrq, nSamples and store in ThisFileInfo
		vector<trace*> traces; // creates a vector of traces
		traces.reserve(nChannels); // preallocate space
		double * data = new double[nChannels*ThisFileInfo.nSamples];
		for(int channel = startChannel; channel < startChannel+nChannels; ++channel){
		    // read a single trace
		  readTrace(filename, &data[(channel-startChannel)*ThisFileInfo.nSamples], channel, ThisFileInfo.nSamples, 0, ThisFileInfo.nSamples);
		    float y = float(channel);
		    traces.push_back(new trace(ThisFileInfo.sampleFrq, ThisFileInfo.nSamples, 0.0, y, &data[(channel-startChannel)*ThisFileInfo.nSamples]));
		    filterProc1(traces.back(),minFrq, maxFrq);
		}

		// cross correlate each receiver against the receiver acting as virtual source
		fftw_complex * xcorrSpec = (fftw_complex *)fftw_malloc( sizeof(fftw_complex)*(1+ThisFileInfo.nSamples/2));
		double * xcorrTime = new double[ThisFileInfo.nSamples];
	       
		for(int channel=startChannel; channel<=endChannel; ++channel){
		  traces[channel-startChannel]->crossCorrelateSpec(*traces[srcChannel-startChannel],xcorrSpec); // cross-corr frq domain
		  fftw_plan inv_plan = fftw_plan_dft_c2r_1d(ThisFileInfo.nSamples,xcorrSpec,xcorrTime,FFTW_ESTIMATE);
		  fftw_execute(inv_plan); // turn frq domain xcorr into time domain
		   // add the cross-correlations onto the xcorr stack 
		   int startPos=0;
		   for(int l=0; l<nLags/2; ++l){
		     xcorrs[l+(channel-startChannel)*nLags] += xcorrTime[l+ThisFileInfo.nSamples-nLags/2];
		   }
		   for(int l=0; l<nLags/2; ++l){
		     xcorrs[(nLags/2)+l+(channel-startChannel)*nLags] += xcorrTime[startPos+l];
		   }
		   fftw_destroy_plan(inv_plan);
		   

		}
		delete[] xcorrTime;
		fftw_free((void *)xcorrSpec);
	   
		for(vector<trace*>::iterator iter=traces.begin(); iter != traces.end(); ++iter){
		  delete (*iter);
		}
		
		delete[] data;
	
	  } // end of while loop that keeps grabbing more data files

	  fileList.close();
	  // record beamform energy for each velocity in the same line of the output text file
	  for(int channel=startChannel; channel<=endChannel; ++channel){

	    for(int lag=0; lag<nLags; ++lag){
	      outfile.write((const char*)(&xcorrs[nLags*(channel-startChannel)+lag]),sizeof(double));
	    }
	  }
	  outfile.close();

      } // end of if statement checking open files

	
  } catch (const std::exception& error){
  	cout << "Exception "<< error.what() << "\n";
  }
  return 0;
}
