import numpy as np
import matplotlib.pyplot as plt
import sys
import matplotlib

infile = sys.argv[1]
#maxVal = float(sys.argv[2])
outfile = sys.argv[2]
minFrq = float(sys.argv[3])
maxFrq = float(sys.argv[4])
outfileEachMin = sys.argv[5]

data = np.loadtxt(infile)
ntimes = data.shape[0]-1
nvels = data.shape[1]

font = {'family' : 'sans-serif',
        'weight' : 'normal',
        'size' : 20}
matplotlib.rc('font',**font)

# first line is list of relevant velocities
velocities = data[0,:]
# rest of lines are actual beamform energies for each velocity and each time
beamformEnergy = data[1:,:]
print(beamformEnergy)

for t in range(ntimes):
    plt.scatter(velocities,beamformEnergy[t,:],s=5,c=np.ones(nvels,dtype=float)*float(t),cmap=plt.cm.cool,linewidths=0,vmin=0, vmax=ntimes-1)
plt.xlabel('N-S velocity (m/s)')
plt.ylabel('beamforming energy/10^6')
plt.show()
plt.clf()

for t in range(ntimes):
    plt.plot(velocities,beamformEnergy[t,:]/1000000,color=(float(t)/float(ntimes),0,1-float(t)/float(ntimes)),linewidth=0.76)
plt.title('Broadband Beamforming')
plt.xlabel('N-S velocity (m/s)')
plt.ylabel('beamforming energy/10^6')
plt.savefig(outfileEachMin)

# stack energy over all time windows
stack = np.zeros((nvels,1))
for t in range(ntimes):
    stack[:,0] = stack[:,0] + beamformEnergy[t,:].T
plt.plot(velocities, stack/1000000)
plt.xlabel('N-S velocity (m/s)')
plt.ylabel('beamforming L1-energy stack/10^6')
plt.title(str(minFrq)+' to '+str(maxFrq)+' Hz')
plt.savefig(outfile)

