#ifndef BEAMFORMING
#define BEAMFORMING

#include "trace.hpp"
#include <math.h>
#include <vector>

void beamform(vector<trace*> traces, double x0, double y0, float xvelocity, float yvelocity, vector<double> &beamformResult){
  // notes about inputs and output *************************

  // initialization of useful variables
  int nT = traces[0]->getNSamples(); // number of time steps
  float dt = 1.0/float(traces[0]->getSampleFrq()); // numer of seconds per sample
  vector<int> avgCounter; // counts how many sensors contribute to each time int he beam forming result 
  avgCounter.reserve(nT);
  for(int t=0; t<nT; ++t){
    beamformResult[t]= 0;
    avgCounter[t] = 0;
  }

  // make sure okay slownesses are used
  float eps = 0.0000001;
  if(std::abs(xvelocity) < eps){
    xvelocity = eps;
  } 
  if(std::abs(yvelocity) < eps){
    yvelocity = eps;
  }

  int nTraces = traces.size();
  for(int tr=0; tr<nTraces; ++tr){
    // shift by distance from center * slowness (then get time index) 
    double dotx = (traces[tr]->getx() - x0)/xvelocity;
    double doty = (traces[tr]->gety() - y0)/yvelocity;
    int rdotu = int((dotx+doty)/dt);
    for(int t=0; t<nT; ++t){
      int sampleIndex = t + rdotu;
      if(sampleIndex < nT && sampleIndex >= 0){
	beamformResult[t] += traces[tr]->getSample(sampleIndex);
	avgCounter[t] += 1;
      }
    }
    
  }
  
  for(int t=0; t<nT; ++t){
    beamformResult[t] /= std::max(1,avgCounter[t]);
  }

}




#endif
