#ifndef FILTER_WORKFLOWS
#define FILTER_WORKFLOWS

#include "trace.hpp"
#include "fftw3.h"
#include "trace_data.hpp"

/*void FK_filter(double * data, double minVelToRemoveMPerSec, double maxVelToRemoveMPerSec, header_vals fileInfo, double channelSpacingMeters,int nChannels,int nSamples){
  
  fftw_complex ***** FKSpec
    fftw_plan fftw_plan_dft_2d(nChannels,nSamples, data, FKSpec, 

}*/

void filterProc1(trace * aTrace, float minFrq, float maxFrq){
  

	// parameters for filtering
	float proportionWhite = 0.5; // only 50% of largest frequencies in band of interest become white
	float nyquistFrq = float(aTrace->getSampleFrq())/2.0; // max frq detected
	
	// despike in small windows
	int samplesInWindow = 50;
	aTrace->despike(samplesInWindow);
	
	// impose 0 mean across the trace
	double theMean = aTrace->getMean();
	aTrace->addConstToTrace(-1.0*theMean);

	// temporal normalization
	aTrace->tempNormalize(minFrq); 

	// whiten
	aTrace->spectralWhiten(proportionWhite, minFrq, maxFrq); // whiten the spectrum
	
	// bandpass to zero below minFrq Hz
	aTrace->zeroSpec(0.0, minFrq); // zero out the spectrum below minFrq Hz
	aTrace->zeroSpec(maxFrq,nyquistFrq); // zero out the spectrum above maxFrq Hz	

	// update time domain ? *****
}


void filterIdentity(trace * aTrace){
	;
}

void hardBandpass(trace * aTrace, float minFrq, float maxFrq){
	// parameters for filtering
	float nyquistFrq = float(aTrace->getSampleFrq())/2.0; // max frq detected
       
	// bandpass to zero below minFrq Hz
	aTrace->zeroSpec(0.0, minFrq); // zero out the spectrum below minFrq Hz
	aTrace->zeroSpec(maxFrq,nyquistFrq); // zero out the spectrum above maxFrq Hz

	// update time domain data
	aTrace->updateTime();
}

#endif
