#include <iostream>
#include <fstream>
#include <string>
#include <assert.h>
#include <vector>
#include <cstdlib>
#include <complex>
#include <math.h>
#include <fftw3.h>
#include "reading.hpp"
#include "trace_data.hpp"
#include "trace.hpp"
using namespace std;

int main(int argc, char* argv[]) {
  try{
    // input file names
    char *fileListName = argv[1]; // input file with list of file names to use
    ifstream fileList(fileListName);
    string filenameStr;
	
    // channels of interest
    int startChannel = atoi(argv[2]);
    int nChannels = atoi(argv[3]); 
    
    // open the output text file where spectral stack results will be printed
    char *outfileName = argv[4];
    ofstream outfile(outfileName,ios::binary|ios::out);
    
    // get basic info about files
    header_vals ThisFileInfo;
    if(fileList.is_open()){
    	streampos startpos = fileList.tellg(); // tell current position in file
    	getline(fileList,filenameStr);
    	const char *filename = filenameStr.c_str();
    	ThisFileInfo = SEGYInfo(filename); // get nTraces, sampleFrq, nSamples
    	fileList.seekg(startpos); // go back to old position in file
    }
    
    // create double array to hold spectral stack
    int nFrq = 1+ThisFileInfo.nSamples/2;
    double *specStack = new double[nChannels*nFrq];
    for(int i=0; i<nChannels*nFrq; ++i){
      specStack[i] = 0;
    }

    if(fileList.is_open() && outfile.is_open()){ 
    
      while(getline(fileList,filenameStr)){	
		// read relevant traces from this file
		const char *filename = filenameStr.c_str();
		cout << filename << "\n";
		//header_vals ThisFileInfo = SEGYInfo(filename); // get nTraces, sampleFrq, nSamples and store in ThisFileInfo
		for(int channel = startChannel; channel < startChannel+nChannels; ++channel){
		    // read a single trace
		    double * oneTrace = new double[ThisFileInfo.nSamples];
		    fftw_complex * spectrum = (fftw_complex *) fftw_malloc( sizeof(fftw_complex)*nFrq);
		    readTrace(filename, oneTrace, channel, ThisFileInfo.nSamples,0, ThisFileInfo.nSamples);
		    // get the spectrum of the trace
		    fftw_plan real_plan;
		    real_plan = fftw_plan_dft_r2c_1d(ThisFileInfo.nSamples, oneTrace, spectrum, FFTW_ESTIMATE);
		    fftw_execute(real_plan);
		    fftw_destroy_plan(real_plan);
		    delete[] oneTrace;
		    // stack this spectrum onto all spectra for this channel
			for(int ifrq=0; ifrq < nFrq; ++ifrq){
				double abs = sqrt(spectrum[ifrq][0]*spectrum[ifrq][0] + spectrum[ifrq][1]*spectrum[ifrq][1]);
				specStack[(channel-startChannel)*nFrq+ifrq] += abs;
			}
		}
      }
      fileList.close();

		
      // record spectrum for each channel
      for(int channel=startChannel; channel < startChannel+nChannels; ++channel){
	     for(int ifrq=0; ifrq < nFrq; ++ifrq){
		    outfile.write((const char*)(&specStack[(channel-startChannel)*nFrq+ifrq]),sizeof(double));
	 	}
       }
       outfile.close();

    }
	
    // cleanup
    delete[] specStack;
	
  } catch (const std::exception& error){
  	cout << "Exception "<< error.what() << "\n";
  }
  return 0;
}
